
/*
 * $Id: CltConnMgr.cc,v 1.11.2.5 1999/12/06 07:30:11 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "CltConnMgr.h"
#include "globals.h"


CltConnMgr::CltConnMgr(): thePortMgr(0), theConnLvlLmt(-1) {
}

CltConnMgr::~CltConnMgr() {
	while (theIdleQueue.count())
		closeIdle(theIdleQueue.firstOut(), ConnCloseStat::ckIdleLocal);
}

void CltConnMgr::configure(const SockOpt &anOpt, RndDistr *aPconnUseLmt, int aConnLvlLmt, int srvCnt) {
	ConnMgr::configure(anOpt, aPconnUseLmt);
	theConnLvlLmt = aConnLvlLmt;

	// determine the maximum number of addresses (for concurrent idle conns)
	const int max = theConnLvlLmt >= 0 ? Min(theConnLvlLmt, srvCnt) : srvCnt;
	theIdleHash.ccAddrMax(max);
}

void CltConnMgr::portMgr(PortMgr *aPortMgr) {
	Assert(aPortMgr && !thePortMgr);
	thePortMgr = aPortMgr;
}

// note: idle connections can be closed on-demand
bool CltConnMgr::atHardConnLimit() const {
	return theConnLvlLmt >= 0 && (theConnLvl-theIdleQueue.count()) >= theConnLvlLmt;
}

Connection *CltConnMgr::get(const NetAddr &addr) {
	ConnHashPos pos;
	Connection *conn = 0;
	if (theIdleHash.find(addr, pos)) {
		conn = theIdleHash.delAt(pos);
		theIdleQueue.dequeue(conn);
		conn->theRd.stop(this);
		TheFileScanner->clearTimeout(conn->sock().fd());
	} else {
		// check if we should close some idle conn first
		if (theConnLvlLmt >= 0 && theConnLvl >= theConnLvlLmt)
			closeIdle(theIdleQueue.firstOut(), ConnCloseStat::ckIdleLocal);
		conn = open(addr);
	}

	if (conn)
		conn->use();

	Assert(theConnLvlLmt < 0 || theConnLvl <= theConnLvlLmt);
	return conn;
}

Connection *CltConnMgr::open(const NetAddr &addr) {
	Connection *conn = TheConnFarm.get();
	if (conn->connect(addr, theSockOpt, thePortMgr)) {
		opened(conn);
		return conn;
	} else {
		// connect failed
		TheConnFarm.put(conn);
		return 0;
	}
}

// called on an idle connection closed by the other side
void CltConnMgr::noteReadReady(int fd) {
	Connection *conn = TheConnIdx[fd];
	Assert(conn);

	// delete connection from the idle queue
	closeIdle(conn, ConnCloseStat::ckIdleForeign);
}

void CltConnMgr::putIdle(Connection *conn) {
	theIdleQueue.enqueue(conn);
	theIdleHash.add(conn);
	ConnMgr::putIdle(conn);
}

void CltConnMgr::closeIdle(Connection *conn, ConnCloseStat::CloseKind ck) {
	Assert(conn);
	theIdleQueue.dequeue(conn);
	theIdleHash.del(conn);
	conn->theRd.stop(this);
	conn->closeKind(ck); 
	close(conn);
}
