
/*
 * $Id: ErrorLx.cc,v 1.1.2.1 1999/11/20 18:10:51 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include <iostream.h>
#include <iomanip.h>

#include "ErrorLx.h"
#include "gadgets.h"

ErrorStrTbl ErrorLxRec::TheStrs;


ErrorLx::ErrorLx(): theExor(0) {
}

bool ErrorLx::configure(const String &aName, const LxParams &params) {
	theName = aName;

	if (theName == "err_cnt_tbl")
		theExor = &ErrorLx::exErrorH;
	else
		return false;

	checkParams(params, 0);

	return theExor;
}

void ErrorLx::extract(const LxRec &r, ostream &os) {
	Assert(theExor);
	// call the pre-configured method
	(this->*theExor)(r.errorStat(), os);
}

void ErrorLx::exErrorH(const ErrorStat &es, ostream &os) const {
	os << endl << "#errno  count count% explanation" << endl;

	const int total = es.totalErrCnt();
	for (int e = 0; e < es.maxErrNo(); ++e) {
		if (const int ecount = es[e]) {
			os 
				<< setw(6) << e
				<< ' ' << setw(6) << ecount
				<< ' ' << setw(6) << Percent(ecount, total)
				<< " \"" << ErrorLxRec::TheStrs[e] << '"' << endl;
		}
	}
}
