
/*
 * $Id: Goal.cc,v 1.6 1999/09/07 17:46:09 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "ILog.h"
#include "OLog.h"
#include "StatPhase.h"
#include "Goal.h"

#include "GoalSym.h"


Goal::Goal() {
}

void Goal::configure(const GoalSym *cfg) {
	Assert(cfg);
	theDuration = cfg->duration();
	cfg->xactCount(theXactCnt);
	cfg->fillSize(theFillSz);
	cfg->errs(theErrs.theRatio, theErrs.theCount);
}

// XXX: magic constant in "theXactCnt >= 1000"
bool Goal::reached(const StatPhase &ph) const {
	return // meeting one sub-goal is enough!
		(theDuration >= 0 && theDuration <= ph.duration()) ||
		(theXactCnt >= 0 && theXactCnt <= ph.xactCnt()) ||
		(theFillSz >= 0 && theFillSz <= ph.fillSz()) ||
		(theErrs.count() >= 0 && theErrs.count() <= ph.errCnt()) ||
		(theXactCnt >= 1000 && theErrs.ratio() >= 0 && theErrs.ratio() <= ph.errRatio())
		;
}
