
/*
 * $Id: IntvlLx.cc,v 1.3.6.3 1999/12/18 19:20:53 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "IntvlLx.h"

IntvlLx::IntvlLx(): theExor(0) {
}

bool IntvlLx::configure(const String &aName, const LxParams &params) {
	theName = aName;

	/* check parametrized objects first */

	if (theName == "rptm_mean" && params.count()) {
		checkParams(params, 1);
		if (params[0] == "hit")
			theExor = &IntvlLx::exHitRptmMean;
		else
		if (params[0] == "miss")
			theExor = &IntvlLx::exMissRptmMean;
		else
		if (params[0] == "all")
			theExor = &IntvlLx::exAllRptmMean;
		else
			badParam(params, 0, "hit, miss, or all");

		return theExor;
	}

	/* the rest of objects are parameterless */

	if (theName == "req_rate")
		theExor = &IntvlLx::exReqRate;
	else
	if (theName == "rep_rate")
		theExor = &IntvlLx::exRepRate;
	else
	if (theName == "conn_lvl_mean")
		theExor = &IntvlLx::exConnLvlMean;
	else
	if (theName == "conn_ttl_mean")
		theExor = &IntvlLx::exConnTtlMean;
	else
	if (theName == "conn_use_mean")
		theExor = &IntvlLx::exConnUseMean;
	else
	if (theName == "req_dhr")
		theExor = &IntvlLx::exReqDHR;
	else
	if (theName == "req_bhr")
		theExor = &IntvlLx::exReqBHR;
	else
	if (theName == "rep_dhr")
		theExor = &IntvlLx::exRepDHR;
	else
	if (theName == "rep_bhr")
		theExor = &IntvlLx::exRepBHR;
	else
	if (theName == "rep_dcr")
		theExor = &IntvlLx::exRepDCR;
	else
	if (theName == "rep_bcr")
		theExor = &IntvlLx::exRepBCR;
	else
	if (theName == "rptm_mean")
		theExor = &IntvlLx::exAllRptmMean; // parameterless case
	else
	if (theName == "rep_sz_mean")              // duplicates rep_sz_hist
		theExor = &IntvlLx::exRepSzMean;
	else
	if (theName == "hit_sz")
		theExor = &IntvlLx::exHitSz;
	else
	if (theName == "miss_sz")
		theExor = &IntvlLx::exMissSz;
	else
	if (theName == "cachable_sz")
		theExor = &IntvlLx::exChbSz;
	else
	if (theName == "uncachable_sz")
		theExor = &IntvlLx::exUnchbSz;
	else
	if (theName == "fill_sz")
		theExor = &IntvlLx::exFillSz;
	else
	if (theName == "rep_cnt")
		theExor = &IntvlLx::exRepCnt;
	else
	if (theName == "hit_cnt")
		theExor = &IntvlLx::exHitCnt;
	else
	if (theName == "miss_cnt")
		theExor = &IntvlLx::exMissCnt;
	else
	if (theName == "cachable_cnt")
		theExor = &IntvlLx::exChbCnt;
	else
	if (theName == "uncachable_cnt")
		theExor = &IntvlLx::exUnchbCnt;
	else
	if (theName == "fill_cnt")
		theExor = &IntvlLx::exFillCnt;
	else
	if (theName == "err_cnt")
		theExor = &IntvlLx::exErrCnt;
	else
	if (theName == "err_pcnt")
		theExor = &IntvlLx::exErrPcnt;
	else
	if (theName == "duration")
		theExor = &IntvlLx::exDuration;
	else
		return false;

	checkParams(params, 0);

	return theExor;
}

void IntvlLx::extract(const LxRec &r, ostream &os) {
	Assert(theExor);
	// call the pre-configured method
	(this->*theExor)(r.intvl(), os);
}
