
/*
 * $Id: LogExtractor.cc,v 1.2.6.1 1999/11/20 18:10:53 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "LogExtractor.h"
#include "opts.h"

/* LxRec */

LxRec::~LxRec() {
}

const StatIntvlRec &LxRec::intvl() const { panic("intvl"); return *(const StatIntvlRec*)0; }
const StatCycleRec &LxRec::cycle() const { panic("cycle"); return *(const StatCycleRec*)0; }
const StatPhaseRec &LxRec::phase() const { panic("phase"); return *(const StatPhaseRec*)0; }
const ErrorStat &LxRec::errorStat() const { panic("errorStat"); return *(const ErrorStat*)0; }

void LxRec::panic(const String &violator) const {
	cerr << here << "extractor requested " << violator 
		<< " from " << type() << " record" << endl;
	Assert(false);
}


/* LxParams */

LxParams::LxParams(const String &src) {
	set(src);
}

LxParams::~LxParams() {
	while (count()) delete pop();
}

int LxParams::count() const {
	return theCount;
}

const String &LxParams::image() const {
	return theImage;
}

const String &LxParams::operator [](int idx) const {
	Assert(idx >= 0 && idx < theCount);
	Assert(item(idx));
	return *item(idx);
}

void LxParams::set(const String &src) {
	String buf = theImage = src;

	while (buf) {
		String head, tail;
		if (splitVal(buf, head, tail, ','))
			buf = tail;
		else
			head = buf, buf = 0;

		append(new String(head));
	}
}


/* LogExtractor */

LogExtractor::LogExtractor() {
}

LogExtractor::~LogExtractor() {
}

void LogExtractor::label(ostream &os) {
	os << theLabel;
}

bool LogExtractor::configure(const String &aName, const LxParams &params) {
	Assert(theName); // must be preset to use default configure
	if (theName != aName)
		return false;
	theLabel = aName;
	checkParams(params, 0);
	return true;
}

void LogExtractor::checkParams(const LxParams &params, int count) {
	if (!count && params.count())
		cerr << "object " << theName << " expects no parameters" << endl;
	else
	if (count != params.count())
		cerr << "object " << theName << " expects " << count << " parameters" << endl;
	else {
		theLabel = theName;
		if (params.count())
			theLabel += ":" + params.image();
		return;
	}

	// dump bad parameters
	cerr << "got " << params.count() << " params: `" << params.image() << "'" << endl;
	exit(-2);
}

void LogExtractor::badParam(const LxParams &params, int idx, const String &exp) const {
	cerr << theName << "'s parameter [" << 1+idx << "] is invalid; "
		<< "got: `" << params[idx] << "' expected: " << exp << endl;
	exit(-2);
}
