
/*
 * $Id: NotifMsg.cc,v 1.4 1999/09/07 17:46:16 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "NetAddr.h"
#include "NotifMsg.h"
#include "gadgets.h"

/* network <-> host byte order conversions */

inline
void hton(Time &t) {
	t.tv_sec = htonl(t.tv_sec);
	t.tv_usec = htonl(t.tv_usec);
}

inline
void ntoh(Time &t) {
	t.tv_sec = ntohl(t.tv_sec);
	t.tv_usec = ntohl(t.tv_usec);
}

inline
void hton(NetDouble &d) {
	d.mnt = (int) htonl(d.mnt);
	d.exp = (int) htonl(d.exp);
}

inline
void ntoh(NetDouble &d) {
	d.mnt = (int) ntohl(d.mnt);
	d.exp = (int) ntohl(d.exp);
}

/* NotifMsg */

void NotifMsg::hton() {
	theId = (int) htonl(theId);
	theSize = (int) htonl(theSize);
}

void NotifMsg::ntoh() {
	theId = (int) ntohl(theId);
	theSize = (int) ntohl(theSize);
}


/* StatusNotifMsg */

StatusNotifMsg::StatusNotifMsg(const String &aLabel): NotifMsg(mtStatus) {
	theSize = sizeof(*this);

	memset(theLabel, 0, sizeof(theLabel));
	if (aLabel)
		strncpy(theLabel, (const char*)aLabel, sizeof(theLabel));
	
	theXactTotCnt = theErrTotCnt = theSockInstCnt = -1;
}

void StatusNotifMsg::hton() {
	NotifMsg::hton();

	::hton(theStartTime);
	::hton(theSndTime);
	::hton(theRespTime);

	::hton(theReqRate);
	::hton(theRepRate);
	::hton(theBwidth);
	::hton(theDHR);
	::hton(theConnUse);
	::hton(theErrRatio);

	theXactTotCnt = (int) htonl(theXactTotCnt);
	theErrTotCnt = (int) htonl(theErrTotCnt);
	theSockInstCnt = (int) htonl(theSockInstCnt);
}

void StatusNotifMsg::ntoh() {
	NotifMsg::ntoh();

	::ntoh(theStartTime);
	::ntoh(theSndTime);
	::ntoh(theRespTime);

	::ntoh(theReqRate);
	::ntoh(theRepRate);
	::ntoh(theBwidth);
	::ntoh(theDHR);
	::ntoh(theConnUse);
	::ntoh(theErrRatio);

	theXactTotCnt = (int) ntohl(theXactTotCnt);
	theErrTotCnt = (int) ntohl(theErrTotCnt);
	theSockInstCnt = (int) ntohl(theSockInstCnt);
}


/* StatusFwdMsg */

StatusFwdMsg::StatusFwdMsg(): theCopyCnt(-1) {
	theSndAddr.addr.s_addr = INADDR_NONE;
	theSndAddr.port = -1;
}

StatusFwdMsg::StatusFwdMsg(const StatusNotifMsg &m, Time aRcvTime, const NetAddr &aSndAddr):
	StatusNotifMsg(m), theRcvTime(aRcvTime), theCopyCnt(-1) {

	theSndAddr.addr = aSndAddr.addrN();
	theSndAddr.port = aSndAddr.port();
}

void StatusFwdMsg::hton() {
	StatusNotifMsg::hton();

	::hton(theRcvTime);

	// addr_in is always in network byte order?
	theSndAddr.port = (int) htonl(theSndAddr.port);

	theCopyCnt = (int) htonl(theCopyCnt);
}

void StatusFwdMsg::ntoh() {
	StatusNotifMsg::ntoh();

	::ntoh(theRcvTime);

	// addr_in is always in network byte order?
	theSndAddr.port = (int) ntohl(theSndAddr.port);

	theCopyCnt = (int) ntohl(theCopyCnt);
}
		
