
/*
 * $Id: Notifier.cc,v 1.2 1999/09/07 17:46:17 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "Clock.h"
#include "LogComment.h"
#include "StatIntvlRec.h"
#include "NotifMsg.h"
#include "Notifier.h"

#include "gadgets.h"


Notifier::Notifier(const String &aLabel, const NetAddr &anAddr):
	theLabel(aLabel), theAddr(anAddr) {

	bool ok = true;
	ok = ok && theSock.create(AF_INET, SOCK_DGRAM, 0);
	ok = ok && theSock.connect(theAddr);
	ok = ok && theSock.blocking(false);

	if (!ok) {
		Comment(2) << "warning: cannot connect to " << theAddr << " to send monitoring messages; possible cause: " << error(errno) << endc;
		theSock.close();
	}
}

Notifier::~Notifier() {
	if (active())
		theSock.close();
}

bool Notifier::active() const {
	return theSock >= 0;
}


void Notifier::noteStats(const StatIntvlRec &s) {
	if (theSock < 0)
		return;
	
	StatusNotifMsg msg(theLabel);

	msg.theStartTime = TheClock.TheStartTime;
	msg.theSndTime = TheClock;
	msg.theRespTime = Time::Msec((int)s.respTime().mean());
	msg.theReqRate = s.reqRate();
	msg.theRepRate = s.repRate();
	msg.theBwidth = Ratio(s.repSz().byted(), s.theDuration.secd());
	msg.theDHR = s.theRealHR.dhr();
	msg.theConnUse = s.theConnUseCnt.mean();
	msg.theErrRatio = s.errRatio();

	msg.theXactTotCnt = TheProgress.xacts();
	msg.theErrTotCnt = TheProgress.errs();
	msg.theSockInstCnt = Socket::Level();

	// write, ignoring errors
	msg.hton();
	theSock.write(&msg, sizeof(msg));
}
