
/*
 * $Id: StatPhaseMgr.cc,v 1.6.6.2 1999/12/27 00:18:08 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include <iomanip.h>
#include <strstream.h>

#include "StatPhase.h"
#include "StatPhaseMgr.h"
#include "polyBcastChannels.h"

StatPhaseMgr::StatPhaseMgr(): thePhase(0), thePhaseIdx(-1) {
}

StatPhaseMgr::~StatPhaseMgr() {
	thePhase = 0;
	while (thePhases.count()) delete thePhases.pop();
}

void StatPhaseMgr::addPhase(StatPhase *ph) {
	Assert(ph);
	Assert(!ph->used());
	thePhases.append(ph);
}

void StatPhaseMgr::start() {
	nextPhase();
}

void StatPhaseMgr::forceFlush() {
	for (int i = 0; i < thePhases.count(); ++i)
		if (thePhases[i]->used())
			thePhases[i]->flush();
}

void StatPhaseMgr::noteDone(StatPhase *ph) {
	Assert(thePhase == ph);
	nextPhase();
}

void StatPhaseMgr::nextPhase() {
	const StatPhase *prevPhase = thePhase;

	if (thePhaseIdx >= thePhases.count() - 1) {
		// all phases have finished
		strstream s;
		s << "all " << thePhases.count() << " phases met their goals" << ends;
		Broadcast(ThePhasesEndChannel, s.str());
		s.freeze(false);

		// add a never-finishing phase when all official phases are over
		// so that agents do not have to check for some phase to exist
		// note: this phase is never deleted, but that's OK
		thePhase = new StatPhase;
		thePhase->name("cold");
		thePhase->statsLogged(false);
	} else {
		thePhase = thePhases[++thePhaseIdx];
	}

	Assert(thePhase);
	thePhase->start(this, prevPhase);
}

void StatPhaseMgr::reportCfg(ostream &os) const {

	// header
	os 
		<< setw(10) << "phase"
		<< ' ' << setw(8) << "load_beg"
		<< ' ' << setw(8) << "load_end"
		<< ' ' << setw(8) << "goal"
		<< "\t\t flags"
		<< endl;

	for (int i = 0; i < thePhases.count(); ++i)
		thePhases[i]->reportCfg(os);
}
