
/*
 * $Id: StatPhaseRec.cc,v 1.9.6.4 1999/12/26 21:38:21 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "StatPhaseRec.h"
#include "OLog.h"
#include "ILog.h"


StatPhaseRec::StatPhaseRec() {
}

OLog &StatPhaseRec::store(OLog &log) const {
	StatIntvlRec::store(log);
	return log 
		<< theName 
		<< theGoal
		<< theConnClose
		<< theSockRdSzH << theSockWrSzH
		<< theRepSzH
		<< theHitRptmH << theMissRptmH
		<< theRptmVsSz << theConnLifeVsUse
		;
}

ILog &StatPhaseRec::load(ILog &log) {
	StatIntvlRec::load(log);
	return log
		>> theName
		>> theGoal
		>> theConnClose
		>> theSockRdSzH >> theSockWrSzH
		>> theRepSzH
		>> theHitRptmH >> theMissRptmH
		>> theRptmVsSz >> theConnLifeVsUse
		;
}

void StatPhaseRec::concat(const StatPhaseRec &r) {
	if (theName && r.theName)
		theName += "|" + r.theName;
	StatIntvlRec::concat(r);
	theGoal.concat(r.theGoal);
	join(r);
}

void StatPhaseRec::merge(const StatPhaseRec &r) {
	if (theName && r.theName)
		theName += "&" + r.theName;
	StatIntvlRec::merge(r);
	theGoal.merge(r.theGoal);
	join(r);
}

void StatPhaseRec::join(const StatPhaseRec &r) {
	if (!theName)
		theName = r.theName;

	theConnClose.add(r.theConnClose);
	theSockRdSzH.add(r.theSockRdSzH);
	theSockWrSzH.add(r.theSockWrSzH);
	theRepSzH.add(r.theRepSzH);
	theHitRptmH.add(r.theHitRptmH);
	theMissRptmH.add(r.theMissRptmH);
	theRptmVsSz.add(r.theRptmVsSz);
	theConnLifeVsUse.add(r.theConnLifeVsUse);
}

ostream &StatPhaseRec::print(ostream &os, const String &pfx) const {
	TimeHist repRptmH;
	repRptmH.add(theHitRptmH);
	repRptmH.add(theMissRptmH);

	os << "name:\t " << theName << endl;

	StatIntvlRec::print(os, pfx);

	theGoal.print(os, pfx + "goal.");
	repRptmH.print(os, pfx + "rep.rptm.");
	theMissRptmH.print(os, pfx + "miss.rptm.");
	theHitRptmH.print(os, pfx + "hit.rptm.");
	theRepSzH.print(os, pfx + "rep.size.");
	theMissRptmH.print(os, pfx + "miss.rptm.");
	theConnClose.print(os, pfx + "conn_close.");
	theSockRdSzH.print(os, pfx + "so_read.size.");
	theSockWrSzH.print(os, pfx + "so_write.size.");
	theRptmVsSz.print(os, pfx + "rptm_vs_size.");
	theConnLifeVsUse.print(os, pfx + "conn_ttl_vs_use.");

	return os;
}
