
/*
 * $Id: ErrorMgr.h,v 1.9 1999/09/17 23:54:37 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_ERROR_MANAGER_H
#define PG_ERROR_MANAGER_H

#include "Array.h"
#include "SrcLoc.h"
#include "ErrorStat.h"

class ErrorStrTbl;

// central point for all error events
// use global wrappers below for error notification
class ErrorMgr {
	public:
		ErrorMgr();

		void configure(const ErrorStrTbl *aStrs);

		// try not to call these directly
		// use global wrappers below instead
		void noteError(int e);
		void reportError(const SrcLoc &loc, int e);
		bool needExpl(int e);

		const ErrorStrTbl *strings() const;
		const ErrorStat &stats() const;

	protected:
		void canonError(int &e) const;

	protected:
		int theVerbLvl;             // how often to explain errors
		const ErrorStrTbl *theStrs; // errno strings

		ErrorStat theCnts;          // error counters
		Array<int> theExplDlines;   // time of next report
		int theTotCnt;              // total number of errors
};

extern ErrorMgr TheErrorMgr;

/* global convenience wrappers */

// count, maybe print a std explanation, return true iff did print
// user may want to provide extra information if returned true
extern bool ReportErrorLoc(const SrcLoc &, int e);

// count, print a std explanation, exit
// user must print any additional info before calling FatalErrorLoc
extern void FatalErrorLoc(const SrcLoc &, int e);


// macros to auto supply source location
#define ReportError(e) ReportErrorLoc(Here, (e))
#define FatalError(e) FatalErrorLoc(Here, (e))

#endif
