
/*
 * $Id: IntIntHash.h,v 1.2 1999/09/17 23:54:38 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_INT_INT_HASH_H
#define PG_INT_INT_HASH_H

#include <limits.h>

// a simple and efficient hash indexed on non-zero 'int'
// the hash stores an 'int' value for each entry
// we will make it more "generic" iff needed

// hash item
struct IntIntHashItem {
	IntIntHashItem *next; // next item in chain
	int key; // position is empty if key is zero
	int val; // initialized to zero

	IntIntHashItem(): next(0), key(0), val(0) {}
};

class IntIntHash {
	public:
		typedef IntIntHashItem **Loc; // an address returned by find() and used in []

	public:
		IntIntHash(int aCapacity); // may be adjusted a bit
		~IntIntHash();

		double utilp() const;

		bool find(int key, Loc &loc) const;
		void addAt(Loc idx, int key, int val);
		void delAt(Loc idx);

		int operator ()(Loc loc) const { return (*loc)->key; }
		int &operator [](Loc loc) { return (*loc)->val; }
		int operator [](Loc loc) const { return (*loc)->val; }

	protected:
		inline int hashIdx(int key) const;

		IntIntHashItem *getNewItem() { return new IntIntHashItem; }
		void putOldItem(IntIntHashItem *i) { delete i; }

	protected:
		Loc theIndex;         // hash (stores pointers to real items)
		int theHashCap;       // hash capacity
		int theHashCnt;       // active pointers in the hash
};


/* inlined methods */

inline
int IntIntHash::hashIdx(int key) const {
	if (key < 0) key += INT_MAX;
	return key % theHashCap;
}

#endif
