
/*
 * $Id: LevelStat.h,v 1.3.6.3 2000/01/13 06:05:53 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef LEVEL_STAT_H
#define LEVEL_STAT_H

#include "Time.h"

class String;
class OLog;
class ILog;

// [concurrency] level statistics
class LevelStat {
	public:
		LevelStat();

		LevelStat &operator ++() { inc(); return *this; }
		LevelStat &operator --() { dec(); return *this; }

		int incCnt() const { return theIncCnt; }
		int decCnt() const { return theDecCnt; }
		int level() const { return theLevel; } 
		double mean() const;

		void restart();
		void store(OLog &log) const;
		void load(ILog &log);

		void keepLevel(const LevelStat &prevLevel);
		void merge(const LevelStat &s);
		void concat(const LevelStat &s);

		ostream &print(ostream &os, const String &pfx) const;

	protected:
		void inc() { change(); theIncCnt++; theLevel++; }
		void dec() { change(); theDecCnt++; theLevel--; }
		void change();

		void join(const LevelStat &s);

	private:
		double nom() const;
		double denom() const;

	protected:
		int theIncCnt;
		int theDecCnt;
		int theLevel;     // internal, not reset on restart

		Time theStart;    // start of measurements
		Time theCurStart; // start of the current level
		Time theSum;      // "level-seconds" (nominator of an average)

	private:
		double theNom;    // these are used for joining
		double theDenom;  // LevelStats only
};


inline OLog &operator <<(OLog &log, const LevelStat &ls) { ls.store(log); return log; }
inline ILog &operator >>(ILog &log, LevelStat &ls) { ls.load(log); return log; }


#endif
