
/*
 * $Id: SrvHistory.h,v 1.3.2.2 1999/11/28 22:43:57 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_SERVER_HISTORY_H
#define PG_SERVER_HISTORY_H

#include "BitMask.h"

class OLog;

// holds information about previously requested objects
// only objects that belong to "public" server area are traced

class SrvHistory {
	public:
		SrvHistory();

		void window(int capacity);

		int newOids(int needCnt, int &resCnt);
		int secOid() { return theSecOid; }

		void noteOid(int oid); // feedback: note requested oid

		void store(OLog &log) const;

	protected:
		int newOid();

		inline void requested(int oid, bool be);
		inline bool requested(int oid) const;

	protected:
		BitMask theAdvWin; // requested advertised oids, sliding window
		int theSecOid;     // last secured oid; all (<= oids) were requested
		int theAdvOid;     // last advertized oid (to some robot)
		int thePendCnt;    // number of not requested advertised oids

		Time theSecTime;   // time when sec oid was updated
};

inline
OLog &operator <<(OLog &log, const SrvHistory &sh) {
	sh.store(log);
	return log;
}

/* inlined methods */

void SrvHistory::requested(int oid, bool be) {
	if (be) {
		theAdvWin.beSet(oid % theAdvWin.capacity());
		--thePendCnt;
	} else {
		theAdvWin.beClear(oid % theAdvWin.capacity());
		++thePendCnt;
	}
}

bool SrvHistory::requested(int oid) const {
	return theAdvWin.isSet(oid % theAdvWin.capacity());
}


#endif
