
/*
 * $Id: StatIntvlRec.h,v 1.7.6.6 2000/01/13 06:05:53 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_STAT_INTERVAL_RECORD_H
#define PG_STAT_INTERVAL_RECORD_H

#include "LogObj.h"
#include "HRStat.h"
#include "AggrStat.h"
#include "LevelStat.h"

// holds basic statistics for a given interval
class StatIntvlRec: public LogObj {
	public:
		StatIntvlRec();

		void restart();
		virtual OLog &store(OLog &log) const;
		virtual ILog &load(ILog &);

		double reqRate() const; // req/sec
		double repRate() const; // rep/sec
		AggrStat respTime() const; // hit + miss

		int complCnt() const { return theXactCnt; }
		BigSize fillSz() const { return theFill.size(); }
		BigSize repSz() const;
		Size repSzMean() const;
		double errRatio() const;
		double errPercent() const;
		Time scanTime() const;
		Time scanStepTime() const;

		void keepLevels(const StatIntvlRec &prevIntvl);
		void concat(const StatIntvlRec &r); // sequential intervals
		void merge(const StatIntvlRec &r);  // concurrent intervals

		ostream &print(ostream &os, const String &pfx) const;

	protected:
		void join(const StatIntvlRec &r);

	public: /* read only, except for kids */
		LevelStat theWaitLvl; // resources waiting requests
		LevelStat theXactLvl; // active xactions
		LevelStat theOpenLvl; // open connections
		LevelStat theEstbLvl; // estanlished connections

		AggrStat theConnLifeTm; // to get mean life time
		AggrStat theConnUseCnt; // xactions per connection

		HRStat theIdealHR;    // offered HR
		HRStat theRealHR;     // measured HR
		HRStat theChbR;       // cachablity ratio

		AggrStat theHitRespTm;   // to get mean response time
		AggrStat theMissRespTm;  // to get mean response time

		CntSzStat theFill;    // cachable misses
		int theXactCnt;       // successful xactions 
		int theXactErrCnt;    // xaction errors

		AggrStat theScanLen;  // sockets scanned during a file scan

		Time theDuration;     // actual intvl length, computed for us
};

#endif
