
/*
 * $Id: AgentSym.cc,v 1.7.6.3 1999/11/28 21:35:23 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "pgl.h"

#include "String.h"
#include "PglRec.h"
#include "PglDistrSym.h"
#include "PglStringSym.h"
#include "SocketSym.h"
#include "AgentSym.h"


AgentSym::AgentSym(const String &aType): HostsBasedSym(aType) {
	theRec->bAdd("Socket", "socket", new SocketSym);

	theRec->bAdd("string", "kind", 0);
	theRec->bAdd("time_distr", "xact_think", 0);
	theRec->bAdd("int_distr", "pconn_use_lmt", 0);
	theRec->bAdd("time", "idle_pconn_tout", 0);
}

AgentSym::AgentSym(const String &aType, PglRec *aRec): HostsBasedSym(aType, aRec) {
}

bool AgentSym::isA(const String &type) const {
	return HostsBasedSym::isA(type) || type == "Agent";
}

SynSym *AgentSym::clone(const String &type) const {
	if (isA(type))
		return new AgentSym(this->type(), theRec->clone());
	return HostsBasedSym::clone(type);
}

String AgentSym::kind() const {
	return getString("kind");
}

SocketSym *AgentSym::socket() const {
	SynSymTblItem *si = 0;
	Assert(theRec->find("socket", si));
	Assert(si->sym());
	return &(SocketSym&)si->sym()->cast("Socket");
}

RndDistr *AgentSym::xactThink() const {
	return getDistr("xact_think");
}

RndDistr *AgentSym::pconnUseLmt() const {
	return getDistr("pconn_use_lmt");
}

Time AgentSym::idlePconnTimeout() const {
	return getTime("idle_pconn_tout");
}
