
/*
 * $Id: PglDistrSym.cc,v 1.4 1999/09/07 17:47:42 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "pgl.h"

#include "Assert.h"
#include "String.h"
#include "Rnd.h"
#include "PglQualifSym.h"
#include "PglDistrSym.h"


// XXX: we are leaking actual distributions as they are never deleted!

DistrSym::DistrSym(const String &aType, RndDistr *aVal): 
	ExpressionSym(aType), theVal(aVal), theQual(0) {
}

DistrSym::~DistrSym() {
	delete theQual;
}

bool DistrSym::isA(const String &type) const {
	return ExpressionSym::isA(type) || type == "Distr";
}

SynSym *DistrSym::clone(const String &type) const {
	if (isA(type)) {
		DistrSym *d = new DistrSym(this->type(), theVal);
		d->qualifier(theQual, theQfOp);
		return d;
	}
	return ExpressionSym::clone(type);
}

void DistrSym::qualifier(const QualifSym *aQual, const String &anOp) {
	delete theQual;
	theQual = aQual ? (QualifSym*)aQual->dup() : 0;
	theQfOp = anOp;
}

ostream &DistrSym::print(ostream &os, const String &) const {
	if (theQual)
		theQual->print(os) << theQfOp;
	if (theVal)
		theVal->print(os);
	else
		os << "<none>";
	return os;
}
