
/*
 * $Id: Map.h,v 1.2 1999/09/07 17:48:40 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef X_MAP_H
#define X_MAP_H

#include "String.h"
#include "Array.h"


// a simple add-only (string -> any) map

// map element
template <class Val>
class MapItem {
	public:
		MapItem();
		MapItem(const String &aKey, const Val &aVal);

	public:
		String *theKey;
		Val theVal;
};

// the map itself
template <class Val>
class Map {
	public:
		typedef MapItem<Val> Item;

	public:
		virtual ~Map();

		int count() const { return theItems.count(); }

		const Item &item(int idx) const { return theItems[idx]; }
		const Item &operator [](int idx) const { return item(idx); }

		const String &keyAt(int idx) const { return *item(idx).theKey; }
		Val &valAt(int idx) { return theItems[idx].theVal; }
		const Val &valAt(int idx) const { return theItems[idx].theVal; }

		void add(const String &key, const Val &val);
		bool find(const String &key, Val &val);
		virtual Val *valp(const String &key);

	protected:
		virtual void noteAdd(int) {} // hook
		virtual void noteDel(int) {} // hook

	protected:
		Array<Item> theItems;
};


/* implementaion of in-lined methods */

/* MapItem */

template <class Val>
inline
MapItem<Val>::MapItem(): theKey(0) {
}

template <class Val>
inline
MapItem<Val>::MapItem(const String &aKey, const Val &aVal):
	theKey(new String(aKey)), theVal(aVal) {
}


/* Map */

template <class Val>
inline
Map<Val>::~Map() {
	while (count()) {
		noteDel(count()-1);
		delete theItems.pop().theKey;
	}
}

template <class Val>
inline
void Map<Val>::add(const String &key, const Val &val) {
	theItems.append(Item(key, val));
	noteAdd(count()-1);
}

template <class Val>
inline
Val *Map<Val>::valp(const String &key) {
	for (int i = 0; i < count(); ++i) {
		if (keyAt(i) == key)
			return &valAt(i);
	}
	return 0;
}

template <class Val>
inline
bool Map<Val>::find(const String &key, Val &v) {
	if (Val *vp = valp(key)) {
		v = *vp;
		return true;
	}
	return false;
}

#endif
